require 'api_service/error_manager'
require 'api_service/failed_result'
require 'api_service/successful_result'

module ApiService
  class ActionState
    attr_reader :response

    def initialize(initial_state = {})
      @response = nil
      @state = initial_state
      @error_manager = ErrorManager.new
    end

    def add_current_action(action, is_last = false)
      @current_action = action
      @is_current_action_last = is_last
    end

    def action
      @current_action
    end

    def valid?
      !errors.any?
    end

    def errors
      @error_manager
    end

    def set(field, value)
      @state[field] = value
    end

    def fetch(field)
      @state[field]
    end

    def build_failed_response
      @response = FailedResult.new(self)
    end

    def success(response = {}, options = {})
      return unless @is_current_action_last
      @response = SuccessfulResult.new(response, self, options)
    end
  end
end
