require 'api_service/action'
require 'api_service/action_state'

module ApiService
  class Base
    class << self
      def actions
        @actions ||= []
      end

      def add_shared_action(name, shared_action, options = {})
        actions << shared_action.new(name, options)
      end

      def add_action(name, options = {}, &action_body)
        actions << Action.new(name, options, action_body)
      end
    end

    def initialize(params, initial_state = {})
      @response = nil
      @params = params
      @state = ActionState.new(initial_state)
    end

    def perform
      actions_pool = self.class.actions
      actions_pool.each_with_index do |action, index|
        @state.add_current_action(action, index == actions_pool.size - 1)

        action.perform(@params, @state)

        if !@state.valid? && action.abort_on_fail?
          @state.build_failed_response
          break
        end
      end

      @state.response
    end
  end
end
