module ApiService
  class ErrorManager
    def initialize
      @errors = {}
    end

    def any?
      @errors.any?
    end

    def add(field, message, scope = '')
      in_scope(scope) do |errors|
        field = field.to_sym
        errors[field] ||= []
        errors[field] << message
        errors
      end
    end

    def to_hash
      @errors
    end

    def from_model(model)
      model.errors.messages.each do |field, errors|
        @errors[field] ||= []
        @errors[field] = @errors[field].concat(errors).uniq
      end
    end

    private

    def in_scope(scope)
      return unless block_given?

      keys = scope.to_s.split('.')
      @errors = yield(@errors) if keys.empty?

      keys.inject(@errors) do |hash, key|
        hash[key] ||= {}
        yield(hash[key]) if key == keys.last
        hash[key]
      end
    end
  end
end
